@@ -1,9 +1,9 @@
 """Moon coverage module."""
 
 from .esa import JUICE_CReMA
-from .maps import CALLISTO, EARTH, EUROPA, GANYMEDE, MOON, IO, VENUS
-from .rois import ROI, GanymedeROIs, CallistoROIs, GeoJsonROI
-from .spice import SPICEPool, SPICERef, et, tdb, utc
+from .maps import CALLISTO, EARTH, EUROPA, GANYMEDE, IO, MOON, VENUS
+from .rois import ROI, CallistoROIs, GanymedeROIs, GeoJsonROI
+from .spice import SpicePool, SpiceRef, et, tdb, utc
 from .trajectory import TourConfig, Trajectory
 from .version import __version__
 
@@ -21,8 +21,8 @@ __all__ = [
     'GeoJsonROI',
     'GanymedeROIs',
     'CallistoROIs',
-    'SPICEPool',
-    'SPICERef',
+    'SpicePool',
+    'SpiceRef',
     'et',
     'tdb',
     'utc',
