@@ -1,12 +1,12 @@
 """CReMA module."""
 
-from pathlib import Path
 from html.parser import HTMLParser
+from pathlib import Path
 from urllib.request import urlopen
 
 from .vars import DATA
 from ..kernels import MetaKernel
-from ..misc import wget, logger
+from ..misc import logger, wget
 
 
 log_esa_crema, debug_esa_crema = logger('ESA CReMA', info_stdout=True)
