@@ -4,7 +4,7 @@ from .crema import CReMACollection
 from .vars import DATA
 
 
-class JUICE_CReMA(metaclass=CReMACollection):  # pylint: disable=R0903
+class JUICE_CReMA(metaclass=CReMACollection):  # pylint: disable=R0903 # noqa:N801
     """JUICE Consolidated Report on Mission Analysis (CReMA).
 
     The complete JUICE Operation SPICE Kernel Dataset can be found here:
