@@ -1,7 +1,7 @@
 """Math module."""
 
 from .greatcircle import great_circle, great_circle_arc
-from .vectors import angle, cs, lonlat, hav_dist, xyz
+from .vectors import angle, cs, hav_dist, lonlat, xyz
 
 
 __all__ = [
