@@ -1,7 +1,8 @@
 """Miscellaneous module."""
 
 from .cache import cached_property, debug_cache
-from .download import wget, debug_download
+from .depreciation import depreciated_renamed
+from .download import debug_download, wget
 from .list import rindex
 from .logger import logger
 from .segment import Segment
@@ -14,5 +15,6 @@ __all__ = [
     'wget',
     'debug_download',
     'cached_property',
+    'depreciated_renamed',
     'debug_cache',
 ]
