@@ -1,7 +1,8 @@
 """Region Of Interest module."""
 
-from .categories import (Category, CategoriesCollection,
-                         SubCategory, SubCategoriesCollection)
+from .categories import (
+    CategoriesCollection, Category, SubCategoriesCollection, SubCategory
+)
 from .geojson import GeoJsonROI
 from .rois import ROI, ROIsCollection, ROIsCollectionWithCategories
 from .rois_stephan_2020_pss import CallistoROIs, GanymedeROIs
