@@ -1,6 +1,6 @@
 """ROIs categories module."""
 
-from .abstract import AbstractItem, AbstractCollection
+from .abstract import AbstractCollection, AbstractItem
 
 
 class Category(AbstractItem):
