@@ -32,6 +32,9 @@ def et(utc, *times):
     if times:
         return [et(t) for t in [utc, *times]]
 
+    if isinstance(utc, (np.int, np.float)):
+        return utc
+
     if isinstance(utc, (tuple, list, np.ndarray)):
         return [et(t) for t in utc]
 
@@ -39,6 +42,34 @@ def et(utc, *times):
     return np.round(sp.str2et(utc), 3)
 
 
+def sclk(sc, utc, *times):
+    """Convert utc time to spacecraft.
+
+    Parametes
+    ---------
+    sc: int
+        NAIF spacecraft ID code.
+    utc: str
+        Input UTC time. All SPICE time formats are accepted.
+
+        For example:
+            - YYYY-MM-DDThh:mm:ss[.ms][Z]
+            - YYYY-MON-DD hh:mm:ss
+
+    *times: str
+        Addition input UTC time(s) to parse.
+
+    """
+    if times:
+        return [sclk(sc, t) for t in [utc, *times]]
+
+    if isinstance(utc, (tuple, list, np.ndarray)):
+        return [sclk(sc, t) for t in utc]
+
+    utc = utc[:-1] if str(utc).endswith('Z') else str(utc)
+    return sp.sce2c(sc, sp.str2et(utc))
+
+
 def utc(et, *ets, unit='ms'):
     """Convert ephemeris time to UTC time(s) in ISOC format.
 
