@@ -4,7 +4,8 @@ import numpy as np
 
 import spiceypy as sp
 
-from .references import SPICERef
+from .limb import limb_ip_pt
+from .references import SpiceBody, SpiceInstrument, SpiceObserver
 from .times import et
 
 
@@ -13,6 +14,29 @@ def is_iter(value):
     return isinstance(value, (list, tuple, np.ndarray))
 
 
+def type_check(value, dtype, func=None):
+    """Check input type.
+
+    Parameters
+    ----------
+    value: any
+        Input value.
+    dtype: type
+        Expected type.
+    func: function, optional
+        Conversion function. Use dtype if `None` provided (default).
+
+    Returns
+    -------
+    dtype
+        Valid value type.
+
+    """
+    return value if isinstance(value, dtype) else \
+        dtype(value) if func is None else \
+        func(value)
+
+
 def deg(rad):
     """Convert radian angle in degrees."""
     return np.multiply(rad, sp.dpr())
@@ -53,8 +77,9 @@ def rlonlat(pt):
 
         xyz = np.divide(pt, big)
     else:
-        xyz = np.zeros_like(pt)
+        xyz = np.zeros_like(pt, dtype=float)
         np.divide(pt, big, where=big > 0, out=xyz, casting='unsafe')
+        xyz[..., np.isnan(big)] = np.nan
 
     radius = big * np.sqrt(np.sum(np.power(xyz, 2), axis=0))
     lat_rad = np.arctan2(xyz[2], np.sqrt(np.sum(np.power(xyz[:2], 2), axis=0)))
@@ -65,6 +90,30 @@ def rlonlat(pt):
     return radius, deg(lon_e_rad) % 360, deg(lat_rad)
 
 
+def radec(vec):
+    """Convert vector on the sky J2000 to RA/DEC coordinates.
+
+    Parameters
+    ----------
+    vec: tuple
+        Input XYZ cartesian vector coordinates i J200 frame.
+
+    Returns
+    -------
+    float or np.ndarray
+        Right-ascension (in degree).
+    float or np.ndarray
+        Declination angle (in degree).
+
+    See Also
+    --------
+    rlonlat
+
+    """
+    _, ra, dec = rlonlat(vec)
+    return ra, dec
+
+
 def sub_obs_pt(time, observer, target, abcorr='NONE', method='NEAR POINT/ELLIPSOID'):
     """Sub-observer point calculation.
 
@@ -79,12 +128,12 @@ def sub_obs_pt(time, observer, target, abcorr='NONE', method='NEAR POINT/ELLIPSO
     abcorr: str, optional
         Aberration correction (default: 'None')
     method: str, optional
-        Computation method to be used.
-        Possible values:
-            - 'NEAR POINT/ELLIPSOID' (default)
-            - 'INTERCEPT/ELLIPSOID'
-            - 'NADIR/DSK/UNPRIORITIZED[/SURFACES = <surface list>]'
-            - 'INTERCEPT/DSK/UNPRIORITIZED[/SURFACES = <surface list>]'
+        Computation method to be used. Possible values:
+
+        - 'NEAR POINT/ELLIPSOID' (default)
+        - 'INTERCEPT/ELLIPSOID'
+        - 'NADIR/DSK/UNPRIORITIZED[/SURFACES = <surface list>]'
+        - 'INTERCEPT/DSK/UNPRIORITIZED[/SURFACES = <surface list>]'
 
         (See NAIF :func:`subpnt` for more details).
 
@@ -102,11 +151,8 @@ def sub_obs_pt(time, observer, target, abcorr='NONE', method='NEAR POINT/ELLIPSO
     spiceypy.subpnt
 
     """
-    if isinstance(time, str):
-        time = et(time)
-
-    if not isinstance(target, SPICERef):
-        target = SPICERef(target)
+    time = type_check(time, float, et)
+    target = type_check(target, SpiceBody)
 
     if is_iter(time):
         return np.transpose([
@@ -114,38 +160,38 @@ def sub_obs_pt(time, observer, target, abcorr='NONE', method='NEAR POINT/ELLIPSO
             for t in time
         ])
 
-    xyz, _, _ = sp.subpnt(method, str(target), time, target.iau_frame,
-                          abcorr, str(observer))
+    xyz, *_ = sp.subpnt(method, str(target), time, target.frame,
+                        abcorr, str(observer))
 
     return xyz
 
 
-def obs_state(time, observer, target, abcorr='NONE'):
-    """Observer position and velocity relative to the target.
+def sc_state(time, spacecraft, target, abcorr='NONE'):
+    """Spacecraft position and velocity relative to the target.
 
-    The position vector starts from the target body to the observer:
+    The position vector starts from the target body to the spacecraft:
 
-        target ------> observer
+        target ------> spacecraft
                 (km)
 
-    The velocity vector correspond to the observer motion (in km/s).
+    The velocity vector correspond to the spacecraft motion (in km/s).
 
     Parameters
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
-        It refers to time at the observer location.
-    observer: str
-        Observer name.
-    target: str
+        It refers to time at the spacecraft location.
+    spacecraft: str or SpiceSpacecraft
+        Spacecraft name.
+    target: str or SpiceBody
         Target body name.
     abcorr: str, optional
         Aberration correction (default: 'None')
 
     Returns
     -------
-    (float, float, float) or np.ndarray
-        Observer XYZ position and velocity coordinates in
+    (float, float, float, float, float, float) or np.ndarray
+        Spacecraft XYZ position and velocity coordinates in
         the target body fixed frame (expressed in km and km/s).
 
         If a list of time were provided, the results will be stored
@@ -161,23 +207,241 @@ def obs_state(time, observer, target, abcorr='NONE'):
     spiceypy.spkezr
 
     """
-    if isinstance(time, str):
-        time = et(time)
-
-    if not isinstance(target, SPICERef):
-        target = SPICERef(target)
+    time = type_check(time, float, et)
+    target = type_check(target, SpiceBody)
 
     if is_iter(time):
         return np.transpose([
-            obs_state(t, observer, target, abcorr=abcorr)
+            sc_state(t, spacecraft, target, abcorr=abcorr)
             for t in time
         ])
 
-    state, _ = sp.spkezr(str(target), time, target.iau_frame,
-                         abcorr, str(observer))
+    state, _ = sp.spkezr(str(target), time, target.frame,
+                         abcorr, str(spacecraft))
     return np.negative(state)
 
 
+def attitude(time, observer, ref='J2000'):
+    """C-matrix attitude.
+
+    Parameters
+    ----------
+    time: float or list or tuple
+        Ephemeris Time or UTC time input(s).
+    observer: str or SpiceSpacecraft or SpiceInstrument
+        Spacecraft or instrument name.
+    ref: str, optional
+        Reference for the return pointing.
+
+    Returns
+    -------
+    np.ndarray
+        C-matrix relative to the reference frame.
+
+        If a list of time were provided, the results will be stored
+        in a (3, 3, N) array.
+
+    Raises
+    ------
+    ValueError
+        If the observer provided is not a Spacecraft or an instrument.
+
+    See Also
+    --------
+    spiceypy.pxform
+
+    """
+    time = type_check(time, float, et)
+    observer = type_check(observer, SpiceObserver)
+
+    if is_iter(time):
+        return np.moveaxis([
+            attitude(t, observer, ref=ref) for t in time
+        ], 0, -1)
+
+    return sp.pxform(str(observer.frame), ref, time)
+
+
+def intersect_pt(time, observer, target, frame, ray, limb=False,
+                 abcorr='NONE', method='ELLIPSOID'):
+    """Intersection point on a target from a ray at the observer position.
+
+    The intersection is primarily computed with the target surface.
+    If no intersection was found and the :py:attr:`limb` flag is set to ``TRUE``,
+    the intersection will be search on the target limb (defined as the impact parameter).
+    When no value was find, a NaN array will be return.
+
+    Parameters
+    ----------
+    time: float or list or tuple
+        Ephemeris Time or UTC time input(s).
+        It refers to time at the observer location.
+    observer: str or SpiceSpacecraft or SpiceInstrument
+        Spacecraft of instrument observer name.
+    target: str or SpiceBody
+        Target body name.
+    frame: str
+        Reference frame relative to which the ray's direction vector is expressed.
+    ray: tuple or list of tuple
+        Ray direction vector emanating from the observer.
+        The intercept with the target body's surface of the ray defined by
+        the observer and ray is sought.
+    limb: bool, optional
+        Compute the intersection on the limb impact parameter
+        if no intersection with the surface was found.
+    abcorr: str, optional
+        Aberration correction (default: 'None')
+    method: str, optional
+        Computation method to be used.
+        (See NAIF :func:`sincpt` for more details).
+
+    Returns
+    -------
+    (float, float, float) or np.ndarray
+        Surface/limb intersection XYZ position on the target body
+        fixed frame (expressed in km).
+
+        If a list of time/ray were provided, the results will be stored
+        in a (3, N) array.
+
+    See Also
+    --------
+    spiceypy.sincpt
+    moon_coverage.spice.limb.limb_ip_pt
+
+    """
+    time = type_check(time, float, et)
+    target = type_check(target, SpiceBody)
+    observer = type_check(observer, SpiceObserver)
+
+    if is_iter(time) and np.ndim(ray) > 1:
+        if len(time) != np.shape(ray)[0]:
+            raise ValueError(
+                'The ephemeris times and ray vectors must have the same size: '
+                f'{len(time)} vs {len(ray)}'
+            )
+
+        return np.transpose([
+            intersect_pt(t, observer, target, frame, r,
+                         limb=limb, abcorr=abcorr, method=method)
+            for t, r in zip(time, ray)
+        ])
+
+    if is_iter(time):
+        return intersect_pt(time, observer, target, frame, [ray] * len(time),
+                            limb=limb, abcorr=abcorr, method=method)
+
+    if np.ndim(ray) > 1:
+        return intersect_pt([time] * np.shape(ray)[0], observer, target, frame, ray,
+                            limb=limb, abcorr=abcorr, method=method)
+
+    try:
+        xyz, *_ = sp.sincpt(method, str(target), time, target.frame, abcorr,
+                            str(observer), frame, ray)
+    except sp.stypes.NotFoundError:
+        if limb:
+            xyz = limb_ip_pt(time, int(observer), int(target), frame, ray, target.frame,
+                             abcorr=abcorr)
+        else:
+            xyz = np.array([np.nan, np.nan, np.nan])
+
+    return xyz
+
+
+def boresight_pt(time, observer, target, limb=False, abcorr='NONE', method='ELLIPSOID'):
+    """Surface intersection on a target from an instrument/spacecraft boresight.
+
+    Parameters
+    ----------
+    time: float or list or tuple
+        Ephemeris Time or UTC time input(s).
+        It refers to time at the observer location.
+    observer: str or SpiceSpacecraft or SpiceInstrument
+        Spacecraft or instrument name.
+    target: str or SpiceBody
+        Target body name.
+    limb: bool, optional
+        Compute the intersection on the limb impact parameter
+        if no intersection with the surface was found.
+    abcorr: str, optional
+        Aberration correction (default: 'None')
+    method: str, optional
+        Computation method to be used.
+        (See NAIF :func:`sincpt` for more details).
+
+    Returns
+    -------
+    (float, float, float) or np.ndarray
+        Boresight intersection XYZ position on the target surface body
+        fixed frame (expressed in km).
+
+        If a list of time were provided, the results will be stored
+        in a (3, N) array.
+
+    See Also
+    --------
+    intersection_pt
+
+    """
+    observer = type_check(observer, SpiceObserver)
+
+    return intersect_pt(time, observer.spacecraft, target, observer.frame,
+                        observer.boresight, limb=limb, abcorr=abcorr, method=method)
+
+
+def fov_pts(time, inst, target, limb=False, npt=24,
+            abcorr='NONE', method='ELLIPSOID'):
+    """Surface intersection on a target from an instrument FOV rays.
+
+    Parameters
+    ----------
+    time: float or list or tuple
+        Ephemeris Time or UTC time input(s).
+        It refers to time at the observer location.
+    inst: str or SpiceInstrument
+        Instrument name.
+    target: str or SpiceBody
+        Target body name.
+    limb: bool, optional
+        Compute the intersection on the limb impact parameter
+        if no intersection with the surface was found.
+    npt: int, optional
+        Number of points in the field of view contour (default: 24).
+    abcorr: str, optional
+        Aberration correction (default: 'None'),
+    method: str, optional
+        Computation method to be used.
+        (See NAIF :func:`sincpt` for more details).
+
+    Returns
+    -------
+    (float, float, float) or np.ndarray
+        Field of View intersection XYZ positions on the target surface body
+        fixed frame (expressed in km).
+
+        If a list of time were provided, the results will be stored
+        in a (3, N, M) array. `M` being the number of bound in the FOV.
+
+    See Also
+    --------
+    intersection_pt
+
+    Note
+    ----
+    In the general case, the last point should be different from the 1st one.
+    You need to add the 1st point to the end of the list if you want to close
+    the polygon of the fooprint.
+
+    """
+    inst = type_check(inst, SpiceInstrument)
+
+    return np.moveaxis([
+        intersect_pt(time, inst.spacecraft, target, inst.frame, ray.copy(),
+                     limb=limb, abcorr=abcorr, method=method)
+        for ray in inst.rays(npt=npt)
+    ], 0, -1)
+
+
 def local_time(time, lon, target, lon_type='PLANETOCENTRIC'):
     """Local solar time.
 
@@ -218,11 +482,8 @@ def local_time(time, lon, target, lon_type='PLANETOCENTRIC'):
     spiceypy.et2lst
 
     """
-    if isinstance(time, str):
-        time = et(time)
-
-    if not isinstance(target, SPICERef):
-        target = SPICERef(target)
+    time = type_check(time, float, et)
+    target = type_check(target, SpiceBody)
 
     if is_iter(time) and is_iter(lon):
         if len(time) != len(lon):
@@ -241,7 +502,10 @@ def local_time(time, lon, target, lon_type='PLANETOCENTRIC'):
     if is_iter(lon):
         return local_time([time] * len(lon), lon, target, lon_type)
 
-    h, m, s, *_ = sp.et2lst(time, int(target), np.radians(lon), lon_type)
+    if not np.isnan(lon):
+        h, m, s, *_ = sp.et2lst(time, int(target), np.radians(lon), lon_type)
+    else:
+        h, m, s = np.nan, np.nan, np.nan
 
     return h + m / 60 + s / 3600
 
@@ -253,6 +517,7 @@ def illum_angles(time, observer, target, pt, abcorr='NONE', method='ELLIPSOID'):
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
+        It refers to time at the surface point.
     observer: str
         Observer name.
     target: str
@@ -287,11 +552,8 @@ def illum_angles(time, observer, target, pt, abcorr='NONE', method='ELLIPSOID'):
     spiceypy.ilumin
 
     """
-    if isinstance(time, str):
-        time = et(time)
-
-    if not isinstance(target, SPICERef):
-        target = SPICERef(target)
+    time = type_check(time, float, et)
+    target = type_check(target, SpiceBody)
 
     if is_iter(time) and np.ndim(pt) > 1:
         if len(time) != np.shape(pt)[1]:
@@ -313,8 +575,11 @@ def illum_angles(time, observer, target, pt, abcorr='NONE', method='ELLIPSOID'):
         return illum_angles([time] * np.shape(pt)[1], observer, target,
                             pt, abcorr=abcorr, method=method)
 
-    *_, p, i, e = sp.ilumin(method, str(target), time,
-                            target.iau_frame, abcorr, str(observer), pt)
+    if not np.isnan(np.max(pt)):
+        *_, p, i, e = sp.ilumin(method, str(target), time,
+                                target.frame, abcorr, str(observer), pt)
+    else:
+        i, e, p = np.nan, np.nan, np.nan
 
     return np.degrees([i, e, p])
 
@@ -351,11 +616,8 @@ def sun_pos(time, target, abcorr='NONE'):
     spiceypy.spkpos
 
     """
-    if isinstance(time, str):
-        time = et(time)
-
-    if not isinstance(target, SPICERef):
-        target = SPICERef(target)
+    time = type_check(time, float, et)
+    target = type_check(target, SpiceBody)
 
     if is_iter(time):
         return np.transpose([
@@ -363,7 +625,7 @@ def sun_pos(time, target, abcorr='NONE'):
             for t in time
         ])
 
-    xyz, _ = sp.spkpos('SUN', time, target.iau_frame, abcorr, str(target))
+    xyz, _ = sp.spkpos('SUN', time, target.frame, abcorr, str(target))
 
     return xyz
 
@@ -394,7 +656,7 @@ def solar_longitude(time, target, abcorr='NONE'):
 
     Note
     ----
-    It the target parent is not the SUN the target will be change
+    If the target parent is not the SUN the target will be change
     for its own parent.
 
     See Also
@@ -402,11 +664,8 @@ def solar_longitude(time, target, abcorr='NONE'):
     spiceypy.lspcn
 
     """
-    if isinstance(time, str):
-        time = et(time)
-
-    if not isinstance(target, SPICERef):
-        target = SPICERef(target)
+    time = type_check(time, float, et)
+    target = type_check(target, SpiceBody)
 
     if target.parent != 'SUN':
         solar_longitude(time, target.parent, abcorr=abcorr)
@@ -455,11 +714,8 @@ def true_anomaly(time, target, abcorr='NONE', frame='ECLIPJ2000'):
     spiceypy.oscltx
 
     """
-    if isinstance(time, str):
-        time = et(time)
-
-    if not isinstance(target, SPICERef):
-        target = SPICERef(target)
+    time = type_check(time, float, et)
+    target = type_check(target, SpiceBody)
 
     if is_iter(time):
         return np.transpose([
@@ -531,8 +787,7 @@ def groundtrack_velocity(target, state):
     spiceypy.mxv
 
     """
-    if not isinstance(target, SPICERef):
-        target = SPICERef(target)
+    target = type_check(target, SpiceBody)
 
     if np.ndim(state) > 1:
         return np.transpose([
