@@ -1,15 +1,21 @@
-"""Trajectory calculation module."""
+"""Trajectory module."""  # pylint: disable=too-many-lines
 
 import numpy as np
 
-from .mask_traj import MaskedTrajectory
+from matplotlib.collections import PathCollection
+from matplotlib.path import Path
 
-from ..math.vectors import angle, norm, ell_norm
+from .mask_traj import MaskedTrajectory
+from ..math.vectors import angle, ell_norm, norm
 from ..misc import cached_property, logger
-from ..spice import SPICEPool, SPICERef, et, et_range, et_ca_range, utc
+from ..spice import (
+    SpiceBody, SpiceInstrument, SpicePool, SpiceRef,
+    SpiceSpacecraft, et, et_ca_range, et_range, utc
+)
 from ..spice.toolbox import (
-    illum_angles, groundtrack_velocity, local_time, obs_state,
-    rlonlat, solar_longitude, sub_obs_pt, sun_pos, true_anomaly
+    attitude, boresight_pt, fov_pts, groundtrack_velocity,
+    illum_angles, local_time, radec, rlonlat, sc_state,
+    solar_longitude, sub_obs_pt, sun_pos, true_anomaly
 )
 
 
@@ -17,17 +23,17 @@ log_traj, debug_trajectory = logger('Trajectory')
 
 
 class Trajectory:  # pylint: disable=too-many-public-methods
-    """Trajectory calculation object.
+    """Spacecraft trajectory object.
 
     Parameters
     ----------
     kernels: str or tuple
         List of kernels to be loaded in the SPICE pool.
 
-    observer: str or spice.SPICERef
-        Observer SPICE reference.
+    observer: str or spice.SpiceSpacecraft or spice.SpiceInstrument
+        Observer (spacecraft or instrument) SPICE reference.
 
-    target: str or spice.SPICERef
+    target: str or spice.SpiceBody
         Target SPICE reference.
 
     ets: float or str or list
@@ -38,6 +44,11 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         the target's position and orientation.
         Only the SPICE keys are accepted.
 
+    Raises
+    ------
+    ValueError
+        If the provided observer is not a valid spacecraft or instrument.
+
     """
 
     TOO_MANY_POINTS_WARNING = 1_000_000
@@ -46,8 +57,8 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         self.kernels = kernels
 
         # Init SPICE references
-        self.observer = SPICERef(observer)
-        self.target = SPICERef(target)
+        self.target = SpiceBody(target)
+        self.observer = observer
 
         # Init ephemeris times
         self.ets = ets
@@ -76,6 +87,39 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """Hat ``^`` operator."""
         return self.mask(self.intersect(other, outside=True))
 
+    def __getitem__(self, item):
+        if item.upper() in ['SUN', 'SS']:
+            return self.ss
+
+        try:
+            # pylint: disable=no-member
+            observer = self.observer.instr(item) \
+                if isinstance(self, SpacecraftTrajectory) else \
+                SpiceRef(item)
+
+            return Trajectory(
+                self.kernels,
+                observer,
+                self.target,
+                self.ets,
+                abcorr='NONE')
+
+        except ValueError:
+            pass
+
+        try:
+            return Trajectory(
+                self.kernels,
+                self.observer,
+                SpiceBody(item),
+                self.ets,
+                abcorr='NONE')
+        except KeyError:
+            pass
+
+        raise KeyError(
+            'The item must be a spacecraft, an instrument, a target body or the SUN. ')
+
     @property
     def kernels(self):
         """Get the list of kernels to used."""
@@ -95,9 +139,33 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """
         self.__kernels = kernels
 
-        if SPICEPool != kernels:
+        if SpicePool != kernels:
             log_traj.info('Load the kernels to the pool.')
-            SPICEPool.add(kernels, purge=True)
+            SpicePool.add(kernels, purge=True)
+
+    @property
+    def observer(self):
+        """Get observer."""
+        return self.__observer
+
+    @observer.setter
+    def observer(self, observer):
+        self.__observer = SpiceRef(observer)
+
+        if isinstance(self.observer, SpiceSpacecraft):
+            self.__class__ = SpacecraftTrajectory
+
+        elif isinstance(self.observer, SpiceInstrument):
+            self.__class__ = InstrumentTrajectory
+
+        else:
+            raise ValueError(f'The observer `{self.observer}` must be a'
+                             '`spacecraft` or an `instrument` reference.')
+
+    @property
+    def spacecraft(self):
+        """Observer spacecraft SPICe reference."""
+        return self.observer.spacecraft
 
     @property
     def ets(self):
@@ -160,14 +228,42 @@ class Trajectory:  # pylint: disable=too-many-public-methods
             raise KeyError(f'Invalid aberration correction key:`{key}`')
         self.__abcorr = key.upper()
 
+    @property
+    def boresight(self):
+        """Observer boresight pointing vector."""
+        return self.observer.boresight
+
+    @boresight.setter
+    def boresight(self, boresight):
+        """Observer boresight setter."""
+        if isinstance(self.observer, SpiceInstrument):
+            raise AttributeError('SpiceInstrument boresight can not be changed')
+
+        log_traj.info('Change boresight from %r to %r.',
+                      self.boresight, boresight)
+
+        # Invalided cached values that require the previous boresight
+        del self.boresight_pts
+        del self.radec
+
+        # Change observer boresight
+        self.observer.BORESIGHT = boresight
+
+    @property
+    def inertial_frame(self):
+        """Target parent inertial frame."""
+        return 'JUICE_JUPITER_IF_J2000' if self.spacecraft == 'JUICE' and \
+            (self.target == 'JUPITER' or self.target.parent == 'JUPITER') else \
+            'ECLIPJ2000'
+
     @cached_property
-    def sub_obs_pts(self):
-        """Sub-observer position vector.
+    def sc_pts(self):
+        """Sub-spacecraft surface intersection vector.
 
         Returns
         -------
         np.ndarray
-            Observer XYZ coordinates in the target body fixed frame
+            Spacecraft XYZ intersection on the target body fixed frame
             (expressed in km).
 
         Note
@@ -186,10 +282,10 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         Trajectory.solar_zenith_angle
 
         """
-        log_traj.info('Compute sub-observer points.')
+        log_traj.info('Compute sub-spacecraft points.')
         res = sub_obs_pt(
             self.ets,
-            self.observer,
+            self.spacecraft,
             self.target,
             abcorr=self.abcorr,
             method='NEAR POINT/ELLIPSOID')
@@ -197,14 +293,14 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         log_traj.debug('Result: %r', res)
         return res
 
-    @cached_property(parent='sub_obs_pts')
-    def sub_obs_rlonlat(self):
-        """Sub-observer coordinates.
+    @cached_property(parent='sc_pts')
+    def sc_rlonlat(self):
+        """Sub-spacecraft coordinates.
 
         Returns
         -------
         np.ndarray
-            Sub-observer planetocentric coordinates:
+            Sub-spacecraft planetocentric coordinates:
             radii, east longitudes and latitudes.
 
         Note
@@ -214,44 +310,80 @@ class Trajectory:  # pylint: disable=too-many-public-methods
 
         See Also
         --------
-        Trajectory.sub_obs_pts
+        Trajectory.sc_pts
         Trajectory.lon_e
         Trajectory.lat
 
         """
         log_traj.debug('Compute sub-observer point in planetocentric coordinates.')
-        return rlonlat(self.sub_obs_pts)
-
-    @property
-    def sub_obs_lon_e(self):
-        """Sub-observer east longitude (degree)."""
-        return self.sub_obs_rlonlat[1]
-
-    @property
-    def sub_obs_lat(self):
-        """Sub-observer latitude (degree)."""
-        return self.sub_obs_rlonlat[2]
+        return rlonlat(self.sc_pts)
 
     @property
     def lonlat(self):
-        """Sub-observer ground track east longitudes and latitudes (degree)."""
-        return self.sub_obs_rlonlat[1:]
+        """Groundtrack or surface intersect (not implemented here)."""
 
-    @cached_property(parent='sub_obs_rlonlat')
-    def local_time(self):
-        """Sub-observer local time (decimal hours).
+    @cached_property
+    def boresight_pts(self):
+        """Boresight surface intersection vector.
+
+        Returns
+        -------
+        np.ndarray
+            Boresight XYZ intersection on the target body fixed frame
+            (expressed in km).
+
+        Note
+        ----
+        Currently the intersection method is fixed internally
+        as ``method='ELLIPSOID'``.
+
+        """
+        log_traj.info('Compute boresight intersection points.')
+        res = boresight_pt(
+            self.ets,
+            self.observer,
+            self.target,
+            abcorr=self.abcorr,
+            method='ELLIPSOID')
+
+        log_traj.debug('Result: %r', res)
+        return res
+
+    @cached_property(parent='boresight_pts')
+    def boresight_rlonlat(self):
+        """Boresight surface intersect coordinates.
+
+        Returns
+        -------
+        np.ndarray
+            Boresight surface intersect planetocentric coordinates:
+            radii, east longitudes and latitudes.
 
         See Also
         --------
-        Trajectory.sub_obs_rlonlat
+        Trajectory.boresight_pts
 
         """
-        log_traj.info('Compute sub-observer local time.')
-        return local_time(self.ets, self.sub_obs_lon_e, self.target)
+        log_traj.debug('Compute boresight intersect in planetocentric coordinates.')
+        return rlonlat(self.boresight_pts)
 
-    @cached_property(parent='sub_obs_pts')
-    def illum_angles(self):
-        """Sub-observer illumination angles (degree).
+    @property
+    def surface(self):
+        """Boresight intersection on the target surface."""
+        return ~np.isnan(self.boresight_pts[0])
+
+    @property
+    def limb(self):
+        """Limb viewing geometry.
+
+        Masked then boresight intersection the target surface.
+
+        """
+        return ~self.surface
+
+    @cached_property(parent='sc_pts')
+    def sc_illum_angles(self):
+        """Sub-spacecraft illumination angles (degree).
 
         Incidence, emission and phase angles.
 
@@ -260,66 +392,57 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         Trajectory.inc
         Trajectory.emi
         Trajectory.phase
-        Trajectory.sub_obs_pts
+        Trajectory.sc_pts
 
         """
-        log_traj.info('Compute sub-observer illumination geometry.')
+        log_traj.info('Compute sub-spacecraft illumination geometry.')
         return illum_angles(
             self.ets,
             self.observer,
             self.target,
-            self.sub_obs_pts,
+            self.sc_pts,
             abcorr=self.abcorr,
             method='ELLIPSOID',
         )
 
-    @property
-    def inc(self):
-        """Sub-observer incidence angle (degree)."""
-        return self.illum_angles[0]
+    @cached_property(parent='boresight_pts')
+    def boresight_illum_angles(self):
+        """Instrument surface intersec illumination angles (degree).
 
-    @property
-    def emi(self):
-        """Sub-observer emission angle (degree)."""
-        return self.illum_angles[1]
+        Incidence, emission and phase angles.
 
-    @property
-    def phase(self):
-        """Sub-observer phase angle (degree)."""
-        return self.illum_angles[2]
-
-    @cached_property(parent='illum_angles')
-    def day(self):
-        """Day side.
-
-        Mask on sub-observer incidence lower than 90°.
+        See Also
+        --------
+        Trajectory.inc
+        Trajectory.emi
+        Trajectory.phase
+        Trajectory.boresight_pts
 
         """
-        return self.inc <= 90  # pylint: disable=comparison-with-callable
-
-    @property
-    def night(self):
-        """Night side.
-
-        Mask on sub-observer incidence larger than 90°.
-
-        """
-        return ~self.day  # pylint: disable=invalid-unary-operand-type
+        log_traj.info('Compute boresight intersection illumination geometry.')
+        return illum_angles(
+            self.ets,
+            self.observer,
+            self.target,
+            self.boresight_pts,
+            abcorr=self.abcorr,
+            method='ELLIPSOID',
+        )
 
     @cached_property
-    def obs_state(self):
+    def sc_state(self):
         """Spacecraft position and velocity in the target body fixed frame (km and km/s).
 
         See Also
         --------
-        Trajectory.obs_pos
+        Trajectory.sc_pos
         Trajectory.alt
 
         """
         log_traj.info('Compute spacecraft position and velocity in the target frame.')
-        res = obs_state(
+        res = sc_state(
             self.ets,
-            self.observer,
+            self.spacecraft,
             self.target,
             abcorr=self.abcorr)
 
@@ -327,8 +450,8 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         return res
 
     @property
-    def obs_pos(self):
-        """Observer position in the target body fixed frame (km).
+    def sc_pos(self):
+        """Spacecraft position in the target body fixed frame (km).
 
         See Also
         --------
@@ -336,82 +459,108 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         Trajectory.alt
 
         """
-        return self.obs_state[:3]
+        return self.sc_state[:3]
 
     @property
-    def obs_velocity(self):
-        """Observer velocity vector in the target body fixed frame (km/s).
+    def sc_velocity(self):
+        """Spacecraft velocity vector in the target body fixed frame (km/s).
 
         See Also
         --------
-        Trajectory.obs_speed
+        Trajectory.sc_speed
 
         """
-        return self.obs_state[3:]
+        return self.sc_state[3:]
 
-    @cached_property(parent='obs_state')
-    def obs_speed(self):
+    @cached_property(parent='sc_state')
+    def sc_speed(self):
         """Observer speed in the target body fixed frame (km/s).
 
         See Also
         --------
-        Trajectory.obs_velocity
+        Trajectory.sc_velocity
 
         """
-        return norm(self.obs_velocity.T)
+        return norm(self.sc_velocity.T)
 
-    @cached_property(parent='obs_state')
+    @cached_property(parent='sc_state')
     def dist(self):
-        """Spacecraft distance to the target (km).
+        """Spacecraft distance to the body target center (km).
 
         This distance is computed to the center of the targeted body.
 
         See Also
         --------
-        Trajectory.obs_pos
+        Trajectory.sc_pos
         Trajectory.alt
+        Trajectory.slant
 
         """
-        log_traj.debug('Compute observer distance in the target frame.')
-        return norm(self.obs_pos.T)
+        log_traj.debug('Compute spacecraft distance in the target frame.')
+        return norm(self.sc_pos.T)
 
-    @cached_property(parent=('sub_obs_pts', 'obs_state'))
+    @cached_property(parent=('sc_pts', 'sc_state'))
     def alt(self):
-        """Spacecraft altitude to the sub-observer point (km).
+        """Spacecraft altitude to the sub-spacecraft point (km).
 
         The intersect on the surface is computed on the reference
         ellipsoide.
 
         See Also
         --------
-        Trajectory.sub_obs_pts
-        Trajectory.obs_pos
+        Trajectory.sc_pts
+        Trajectory.sc_pos
         Trajectory.dist
 
         """
-        log_traj.debug('Compute observer distance in the target frame.')
-        return norm(self.obs_pos.T - self.sub_obs_pts.T)
+        log_traj.debug('Compute spacecraft distance in the target frame.')
+        return norm(self.sc_pos.T - self.sc_pts.T)
 
-    @cached_property(parent='obs_state')
+    @cached_property(parent='sc_state')
     def groundtrack_velocity(self):
         """Compute the groundtrack velocity (km/s).
 
-        It correspond to the motion speed of the sub-observer point
+        It correspond to the motion speed of the sub-spacecraft point
         on the surface.
 
         See Also
         --------
-        Trajectory.obs_state
-        Trajectory.obs_pos
-        Trajectory.obs_velocity
+        Trajectory.sc_state
+        Trajectory.sc_pos
+        Trajectory.sc_velocity
 
         """
-        log_traj.info('Compute ground track velocity.')
-        res = groundtrack_velocity(self.target, self.obs_state)
+        log_traj.info('Compute groundtrack velocity.')
+        res = groundtrack_velocity(self.target, self.sc_state)
 
         log_traj.debug('Result: %r', res)
         return res
 
+    @cached_property
+    def sc_attitude(self):
+        """Spacecraft attitude c-matrix in J2000 frame."""
+        log_traj.info('Compute spacecraft attitude.')
+        return attitude(self.ets, self.observer)
+
+    @cached_property(parent='sc_attitude')
+    def radec(self):
+        """Boresight RA/DEC pointing in J2000 frame."""
+        log_traj.info('Compute spacecraft pointing (RA/DEC).')
+        return radec(np.einsum('ijk,j->ik',
+                               self.sc_attitude,
+                               self.boresight,
+                               optimize=True))
+
+    @property
+    def ra(self):
+        """Boresight pointing right ascension angle (degree)."""
+        return self.radec[0]
+
+    @property
+    def dec(self):
+        """Boresight pointing declination angle (degree)."""
+        return self.radec[1]
+
     @cached_property
     def sun_pos(self):
         """Sun position in the target body fixed frame (km).
@@ -473,7 +622,7 @@ class Trajectory:  # pylint: disable=too-many-public-methods
 
     @property
     def sun_lonlat(self):
-        """Sub-solar point ground track (degree)."""
+        """Sub-solar point groundtrack (degree)."""
         return self.sun_rlonlat[1:]
 
     @property
@@ -481,45 +630,6 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """Alias on the sub-solar point groundtrack (degree)."""
         return self.sun_lonlat
 
-    @cached_property(parent='sub_obs_pts')
-    def ell_norm(self):
-        """Sub-observer local normal.
-
-        Unitary vector pointing upward from the surface of the ellipsoid.
-
-        See Also
-        --------
-        Trajectory.sub_obs_pts
-
-        """
-        log_traj.debug('Compute sub-observer local normal.')
-        res = ell_norm(self.sub_obs_pts.T, self.target.radii).T
-
-        log_traj.debug('Result: %r', res)
-        return res
-
-    @cached_property(parent=('sub_obs_pts', 'sun_pos', 'ell_norm'))
-    def solar_zenith_angle(self):
-        """Sub-observer solar zenith angle (degree).
-
-        The angle is computed from the local normal
-        on the ellipsoid. If the targeted body is a sphere,
-        this value much be equal to the incidence angle.
-
-        See Also
-        --------
-        Trajectory.sun_pos
-        Trajectory.sub_obs_pts
-        Trajectory.ell_norm
-
-        """
-        log_traj.debug('Compute sub-observer solar zenith angle.')
-        res = angle(self.sun_pos.T - self.sub_obs_pts.T,
-                    self.ell_norm.T)  # pylint: disable=no-member
-
-        log_traj.debug('Result: %r', res)
-        return res
-
     @cached_property
     def solar_longitude(self):
         """Target seasonal solar longitude (degrees).
@@ -540,13 +650,6 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         log_traj.debug('Result: %r', res)
         return res
 
-    @property
-    def inertial_frame(self):
-        """Target parent inertial frame."""
-        return 'JUICE_JUPITER_IF_J2000' \
-            if self.target.parent == 'JUPITER' and self.observer == 'JUICE' else \
-            'ECLIPJ2000'
-
     @cached_property
     def true_anomaly(self):
         """Target orbital true anomaly (degree)."""
@@ -601,8 +704,8 @@ class Trajectory:  # pylint: disable=too-many-public-methods
     def approx_ca_utc(self):
         """List of approximate closest approach UTC times to the target.
 
-        Warning
-        -------
+        Danger
+        ------
         These values may not be accurate depending on the
         :py:class:`Trajectory` temporal resolution.
 
@@ -622,6 +725,302 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         ]
 
 
+class SpacecraftTrajectory(Trajectory):
+    """Spacecraft trajectory object."""
+
+    @property
+    def lon_e(self):
+        """Sub-spacecraft east longitude (degree)."""
+        return self.sc_rlonlat[1]
+
+    @property
+    def lat(self):
+        """Sub-spacecraft latitude (degree)."""
+        return self.sc_rlonlat[2]
+
+    @property
+    def lonlat(self):
+        """Sub-spacecraft groundtrack east longitudes and latitudes (degree)."""
+        return self.sc_rlonlat[1:]
+
+    @cached_property(parent='sc_rlonlat')
+    def local_time(self):
+        """Sub-spacecraft local time (decimal hours).
+
+        See Also
+        --------
+        Trajectory.rlonlat
+
+        """
+        log_traj.info('Compute sub-spacecraft local time.')
+        return local_time(self.ets, self.lon_e, self.target)
+
+    @property
+    def slant(self):
+        """Spacecraft line-of-sight distance to the sub-spacecraft point (km).
+
+        This is not the distance the distance to the target body center,
+        but an alias of the altitude of the spacecraft.
+
+        See Also
+        --------
+        Trajectory.dist
+        Trajectory.alt
+
+        """
+        return self.alt
+
+    @property
+    def inc(self):
+        """Sub-spacecraft incidence angle (degree)."""
+        return self.sc_illum_angles[0]
+
+    @property
+    def emi(self):
+        """Sub-spacecraft emission angle (degree)."""
+        return self.sc_illum_angles[1]
+
+    @property
+    def phase(self):
+        """Sub-spacecraft phase angle (degree)."""
+        return self.sc_illum_angles[2]
+
+    @cached_property(parent='sc_illum_angles')
+    def day(self):
+        """Day side, sub-spacecraft incidence lower than 90°."""
+        return self.inc <= 90
+
+    @cached_property(parent='sc_illum_angles')
+    def night(self):
+        """Night side, sub-spacecraft incidence larger than 90°."""
+        return self.inc > 90
+
+    @cached_property(parent='sc_illum_angles')
+    def nadir(self):
+        """Nadir viewing, always True for the sub-spacecraft point."""
+        return np.full_like(self.ets, True, dtype=bool)
+
+    @cached_property(parent='sc_illum_angles')
+    def off_nadir(self):
+        """Off-nadir viewing, always False for the sub-spacecraft point."""
+        return np.full_like(self.ets, False, dtype=bool)
+
+    @cached_property(parent='sc_pts')
+    def ell_norm(self):
+        """Sub-spacecraft local normal.
+
+        Unitary vector pointing upward from the surface of the ellipsoid.
+
+        See Also
+        --------
+        Trajectory.sc_pts
+
+        """
+        log_traj.debug('Compute sub-observer local normal.')
+        res = ell_norm(self.sc_pts.T, self.target.radii).T
+
+        log_traj.debug('Result: %r', res)
+        return res
+
+    @cached_property(parent=('sc_pts', 'sun_pos', 'ell_norm'))
+    def solar_zenith_angle(self):
+        """Sub-spacecraft solar zenith angle (degree).
+
+        The angle is computed from the local normal
+        on the ellipsoid. If the targeted body is a sphere,
+        this value much be equal to the incidence angle.
+
+        See Also
+        --------
+        Trajectory.sun_pos
+        Trajectory.sc_pts
+        Trajectory.ell_norm
+
+        """
+        log_traj.debug('Compute sub-observer solar zenith angle.')
+        res = angle(self.sun_pos.T - self.sc_pts.T,
+                    self.ell_norm.T)  # pylint: disable=no-member
+
+        log_traj.debug('Result: %r', res)
+        return res
+
+
+class InstrumentTrajectory(Trajectory):
+    """Instrument trajectory object."""
+    FOV_PTS = 25
+
+    @property
+    def lon_e(self):
+        """Instrument surface intersect east longitude (degree)."""
+        return self.boresight_rlonlat[1]
+
+    @property
+    def lat(self):
+        """Instrument surface intersect latitude (degree)."""
+        return self.boresight_rlonlat[2]
+
+    @property
+    def lonlat(self):
+        """Instrument surface intersect east longitudes and latitudes (degree)."""
+        return self.boresight_rlonlat[1:]
+
+    @cached_property(parent='boresight_rlonlat')
+    def local_time(self):
+        """Instrument surface intersec local time (decimal hours)."""
+        log_traj.info('Compute boresight intersection local time.')
+        return local_time(self.ets, self.lon_e, self.target)
+
+    @cached_property(parent=('sc_pts', 'boresight_pts'))
+    def slant(self):
+        """Line-of-sight distance to the boresight surface intersection (km)."""
+        log_traj.debug('Compute slant range to the boresight surface intersection.')
+        return norm(self.sc_pos.T - self.boresight_pts.T)
+
+    @property
+    def inc(self):
+        """Instrument surface intersec incidence angle (degree)."""
+        return self.boresight_illum_angles[0]
+
+    @property
+    def emi(self):
+        """Instrument surface intersec emission angle (degree)."""
+        return self.boresight_illum_angles[1]
+
+    @property
+    def phase(self):
+        """Instrument surface intersec phase angle (degree)."""
+        return self.boresight_illum_angles[2]
+
+    @cached_property(parent='boresight_illum_angles')
+    def day(self):
+        """Day side, boresight intersection incidence lower than 90°."""
+        return self.inc <= 90
+
+    @cached_property(parent='boresight_illum_angles')
+    def night(self):
+        """Night side, boresight intersection incidence larger than 90°."""
+        return self.inc > 90
+
+    @cached_property(parent='boresight_illum_angles')
+    def nadir(self):
+        """Nadir viewing, boresight intersection emission angle is lower than 1°."""
+        return self.emi < 1
+
+    @cached_property(parent='boresight_illum_angles')
+    def off_nadir(self):
+        """Off-nadir viewing, boresight intersection emission angle is larger than 1°."""
+        return self.emi >= 1
+
+    @cached_property(parent='boresight_pts')
+    def ell_norm(self):
+        """Instrument surface intersec local normal.
+
+        Unitary vector pointing upward from the surface of the ellipsoid.
+
+        See Also
+        --------
+        Trajectory.boresight_pts
+
+        """
+        log_traj.debug('Compute sub-observer local normal.')
+        res = ell_norm(self.boresight_pts.T, self.target.radii).T
+
+        log_traj.debug('Result: %r', res)
+        return res
+
+    @cached_property(parent=('boresight_pts', 'sun_pos', 'ell_norm'))
+    def solar_zenith_angle(self):
+        """Instrument surface intersec solar zenith angle (degree).
+
+        The angle is computed from the local normal
+        on the ellipsoid. If the targeted body is a sphere,
+        this value much be equal to the incidence angle.
+
+        See Also
+        --------
+        Trajectory.sun_pos
+        Trajectory.boresight_pts
+        Trajectory.ell_norm
+
+        """
+        log_traj.debug('Compute sub-observer solar zenith angle.')
+        res = angle(self.sun_pos.T - self.boresight_pts.T,
+                    self.ell_norm.T)  # pylint: disable=no-member
+
+        log_traj.debug('Result: %r', res)
+        return res
+
+    @cached_property
+    def fov_pts(self):
+        """Instrument FOV points.
+
+        Note
+        ----
+        Currently the intersection method is fixed internally
+        as ``method='ELLIPSOID'``.
+
+        """
+        log_traj.debug('Compute FOV intersection points.')
+        res = fov_pts(
+            self.ets,
+            self.observer,
+            self.target,
+            limb=True,
+            npt=self.FOV_PTS - 1,
+            abcorr=self.abcorr,
+            method='ELLIPSOID')
+
+        # Close the polygon
+        res = np.dstack([res, res[..., 0]])
+
+        log_traj.debug('Result: %r', res)
+        return res
+
+    @cached_property(parent='fov_pts')
+    def fov_rlonlat(self):
+        """Instrument FOV intersect coordinates.
+
+        Returns
+        -------
+        np.ndarray
+            Boresight surface intersect planetocentric coordinates:
+            radii, east longitudes and latitudes.
+
+        See Also
+        --------
+        Trajectory.fov_pts
+
+        """
+        log_traj.debug('Compute FOV intersects planetocentric coordinates.')
+        return rlonlat(self.fov_pts)
+
+    @cached_property(parent='fov_rlonlat')
+    def fov_paths(self):
+        """Instrument FOV surface paths.
+
+        Note
+        ----
+        If all the points are above the limb the path is set to ``None``.
+
+        See Also
+        --------
+        Trajectory.fov_rlonlat
+
+        """
+        log_traj.debug('Compute FOV paths.')
+        r_max = max(self.target.radii)
+
+        return [
+            Path(rlonlat[..., 1:]) if np.min(rlonlat[..., 0]) < r_max else None
+            for rlonlat in np.moveaxis(self.fov_rlonlat, 0, -1)
+        ]
+
+    def fovs(self, label=None, **kwargs):
+        """Instrument field of view paths collection."""
+        label = str(self.observer).replace('_', ' ') if label is None else label
+        return PathCollection(self.fov_paths, label=label, **kwargs)
+
+
 class Flyby(Trajectory):
     """Trajectory flyby object.
 
@@ -654,10 +1053,10 @@ class Flyby(Trajectory):
     kernels: str or tuple
         List of kernels to be loaded in the SPICE pool.
 
-    observer: str or spice.SPICERef
+    observer: str or spice.SpiceSpacecraft
         Observer SPICE reference.
 
-    target: str or spice.SPICERef
+    target: str or spice.SpiceBody
         Target SPICE reference.
 
     approx_ca_utc: float, string or numpy.datetime64
@@ -691,7 +1090,7 @@ class Flyby(Trajectory):
 
     def __init__(self, kernels, observer, target, approx_ca_utc, *dt, abcorr='NONE'):
         # Check the pool coverage
-        et_min, et_max = SPICEPool.coverage(observer, target, fmt='ET')
+        et_min, et_max = SpicePool.coverage(observer, target, fmt='ET')
 
         # Coarse stage
         coarse_ets = et_ca_range(approx_ca_utc, (24, 'h', '20 min'))
@@ -742,6 +1141,11 @@ class Flyby(Trajectory):
 
         super().__init__(kernels, observer, target, ets)
 
+        if isinstance(self, SpacecraftTrajectory):
+            self.__class__ = SpacecraftFlyby
+        else:
+            self.__class__ = InstrumentFlyby
+
     def __repr__(self):
         return (
             f'<{self.__class__.__name__}> '
@@ -753,6 +1157,17 @@ class Flyby(Trajectory):
             f'\n - Nb of pts: {len(self):,d}'
         )
 
+    def __getitem__(self, item):
+        traj = super().__getitem__(item)
+
+        if isinstance(traj, SpacecraftTrajectory):
+            traj.__class__ = SpacecraftFlyby
+
+        elif isinstance(traj, InstrumentTrajectory):
+            traj.__class__ = InstrumentFlyby
+
+        return traj
+
     @property
     def duration(self):
         """Flyby duration."""
@@ -785,13 +1200,13 @@ class Flyby(Trajectory):
 
     @property
     def lon_e_ca(self):
-        """Sub-observer west longitude at closest approach."""
-        return self.sub_obs_lon_e[self._i_ca]
+        """Sub-spacecraft west longitude at closest approach."""
+        return self.lonlat[0][self._i_ca]
 
     @property
     def lat_ca(self):
-        """Sub-observer latitude at closest approach."""
-        return self.sub_obs_lat[self._i_ca]
+        """Sub-spacecraft latitude at closest approach."""
+        return self.lonlat[1][self._i_ca]
 
     @property
     def alt_ca(self):
@@ -860,6 +1275,14 @@ class Flyby(Trajectory):
         return start, stop, (stop - start).item()
 
 
+class SpacecraftFlyby(Flyby, SpacecraftTrajectory):
+    """Spacecraft flyby object."""
+
+
+class InstrumentFlyby(Flyby, InstrumentTrajectory):
+    """Instrument flyby object."""
+
+
 class AltitudeTooLowError(Exception):
     """Min value is too low and never reach during the flyby."""
 
